@@ -1,8 +1,8 @@
-import logging
 import os
 from io import BytesIO
 
 import pytest
+
 from PIL import Image, TiffImagePlugin
 from PIL.TiffImagePlugin import RESOLUTION_UNIT, X_RESOLUTION, Y_RESOLUTION
 
@@ -16,8 +16,6 @@ from .helper import (
     is_win32,
 )
 
-logger = logging.getLogger(__name__)
-
 
 class TestFileTiff:
     def test_sanity(self, tmp_path):
@@ -196,7 +194,7 @@ class TestFileTiff:
     def test_save_unsupported_mode(self, tmp_path):
         im = hopper("HSV")
         outfile = str(tmp_path / "temp.tif")
-        with pytest.raises(IOError):
+        with pytest.raises(OSError):
             im.save(outfile)
 
     def test_little_endian(self):
@@ -227,8 +225,8 @@ class TestFileTiff:
             assert im.getpixel((0, 1)) == 0
 
     def test_12bit_rawmode(self):
-        """ Are we generating the same interpretation
-        of the image as Imagemagick is? """
+        """Are we generating the same interpretation
+        of the image as Imagemagick is?"""
 
         with Image.open("Tests/images/12bit.cropped.tif") as im:
             # to make the target --
@@ -249,7 +247,7 @@ class TestFileTiff:
             assert im.getextrema() == (-3.140936851501465, 3.140684127807617)
 
     def test_unknown_pixel_mode(self):
-        with pytest.raises(IOError):
+        with pytest.raises(OSError):
             Image.open("Tests/images/hopper_unknown_pixel_mode.tif")
 
     def test_n_frames(self):
@@ -387,6 +385,10 @@ class TestFileTiff:
         ret = ifd.load_double(data, False)
         assert ret == (8.540883223036124e194, 8.540883223036124e194)
 
+    def test_ifd_tag_type(self):
+        with Image.open("Tests/images/ifd_tag_type.tiff") as im:
+            assert 0x8825 in im.tag_v2
+
     def test_seek(self):
         filename = "Tests/images/pil136.tiff"
         with Image.open(filename) as im:
@@ -614,7 +616,7 @@ class TestFileTiffW32:
         im = Image.open(tmpfile)
         fp = im.fp
         assert not fp.closed
-        with pytest.raises(WindowsError):
+        with pytest.raises(OSError):
             os.remove(tmpfile)
         im.load()
         assert fp.closed
